import traverse from '@babel/traverse';
import * as t from '@babel/types';
import { parse } from '@babel/parser';
import { transformFromAstSync } from '@babel/core';
import { transformJS } from './transform';
import { defaultBabelConfig } from '../config/default-babel-config';
import { transformAst } from './ast';
function extractVariableNames(pattern) {
  var names = [];
  var _traverse = function traverse(node) {
    if (!node) return;
    switch (node.type) {
      case 'Identifier':
        names.push(node.name);
        break;
      case 'ArrayPattern':
        node.elements.forEach(_traverse);
        break;
      case 'ObjectPattern':
        node.properties.forEach(function (prop) {
          if (prop.type === 'ObjectProperty') {
            // 处理 { a: b } 形式
            if (prop.value.type === 'Identifier') {
              names.push(prop.value.name);
            } else {
              _traverse(prop.value);
            }
          }
          // 处理剩余运算符 ...rest
          else if (prop.type === 'RestElement') {
            _traverse(prop.argument);
          }
        });
        break;
      case 'AssignmentPattern':
        _traverse(node.left);
        break;
      case 'RestElement':
        _traverse(node.argument);
        break;
    }
  };
  _traverse(pattern);
  return names;
}

/**
 * 构造一个本地名→{ package, importName } 的映射
 */
/** 构建本地名→{ package, importName } 映射 */
function buildImportMap(ast) {
  var importMap = {};
  traverse(ast, {
    ImportDeclaration: function ImportDeclaration(path) {
      var pkg = path.node.source.value;
      path.node.specifiers.forEach(function (spec) {
        if (t.isImportSpecifier(spec)) {
          importMap[spec.local.name] = {
            "package": pkg,
            importName: spec.imported.name
          };
        } else if (t.isImportDefaultSpecifier(spec)) {
          importMap[spec.local.name] = {
            "package": pkg,
            importName: spec.local.name,
            "default": true
          };
        } else if (t.isImportNamespaceSpecifier(spec)) {
          importMap[spec.local.name] = {
            "package": pkg,
            importName: spec.local.name,
            namespace: true
          };
        }
      });
    }
  });
  return importMap;
}

/** 在给定的语句路径上收集“真正的”import依赖 */
function collectDeps(stmtPath, importMap) {
  var deps = {};
  stmtPath.traverse({
    Identifier: function Identifier(idPath) {
      var name = idPath.node.name;
      // 只有当这个 name 在 importMap 中，
      // 并且当前 scope 里确实把它当做 “module” 绑定时，
      // 才算是对 import 的引用
      var binding = idPath.scope.getBinding(name);
      if (importMap[name] && binding && binding.kind === 'module') {
        var key = importMap[name]["package"] + "::" + importMap[name].importName;
        deps[key] = importMap[name];
      }
    }
  });

  // 返回去重后的数组
  return Object.values(deps);
}

/**
 * 解析代码并提取结构化语句信息
 * @param code 原始代码字符串
 * @param ast 经过转换的AST语法树
 * @returns 包含语句集合和错误信息的对象
 */
export var statementParser = function statementParser(code, ast) {
  var statements = [];
  var errorsByStatements = {};
  // 第一步：拿到 importMap
  var importMap = buildImportMap(ast);
  traverse(ast, {
    FunctionDeclaration: function FunctionDeclaration(path) {
      var _path$node$id;
      if (((_path$node$id = path.node.id) === null || _path$node$id === void 0 ? void 0 : _path$node$id.name) !== 'Page') return;
      path.get('body.body').forEach(function (stmtPath) {
        var stmt = stmtPath.node;
        var snippet = code.slice(stmt.start, stmt.end);
        // 针对这一条 stmtPath 收集真正的依赖
        var dependencies = collectDeps(stmtPath, importMap);

        // 编译检测
        var tmpAst = parse('');
        tmpAst.program.body.push(stmt);
        var transformed = transformFromAstSync(tmpAst);
        var _transformJS = transformJS(transformed.code, defaultBabelConfig),
          hasError = _transformJS.hasError,
          errorInfo = _transformJS.errorInfo,
          _transformJS$code = _transformJS.code,
          compiledCode = _transformJS$code === void 0 ? '' : _transformJS$code;

        // 变量声明
        if (t.isVariableDeclaration(stmt)) {
          var decl = stmt.declarations[0];
          var names = extractVariableNames(decl.id);
          statements.push({
            type: 'JsVarDeclaration',
            value: compiledCode,
            names: names,
            dependencies: dependencies,
            source: snippet
          });
          if (hasError && errorInfo) {
            errorsByStatements[snippet] = errorInfo;
          }
          return;
        }

        // 表达式语句
        if (t.isExpressionStatement(stmt)) {
          statements.push({
            type: 'JSExpression',
            value: compiledCode,
            dependencies: dependencies,
            source: snippet
          });
          if (hasError && errorInfo) {
            errorsByStatements[snippet] = errorInfo;
          }
          return;
        }

        // 函数声明
        if (t.isFunctionDeclaration(stmt)) {
          var name = stmt.id.name;
          statements.push({
            type: 'JSFunction',
            name: name,
            value: compiledCode,
            dependencies: dependencies,
            source: snippet
          });
          if (hasError && errorInfo) {
            errorsByStatements[snippet] = errorInfo;
          }
        }
      });
    }
  });
  return {
    statements: statements,
    errorsByStatements: errorsByStatements
  };
};

/**
 * 将原始代码转换为结构化语句信息
 * @param code 需要解析的代码字符串
 * @returns 包含语句集合和错误信息的对象
 */
export var parseCodeStatements = function parseCodeStatements(code, imports) {
  var finalCode = imports + "; function Page() { " + code + " }";
  var ast = transformAst(finalCode);
  return statementParser(finalCode, ast);
};